package de.hamster.workbench;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Locale;
import java.util.Properties;

import javax.swing.JMenuItem;
import javax.swing.JFrame;

import de.hamster.compiler.controller.CompilerController;
import de.hamster.debugger.controller.DebuggerController;
import de.hamster.debugger.model.DebuggerModel;
import de.hamster.editor.controller.EditorController;
import de.hamster.model.HamsterFile;
import de.hamster.simulation.controller.SimulationController;
import de.hamster.simulation.view.DialogTerminal;

/**
 * Diese Klasse implementiert den Controller der zentralen Werkbank. Die
 * dazugehoerige main-Methode wird beim Programmstart aufgerufen und erzeugt
 * eine Instanz des Controller. Der Controller erzeugt dann Instanzen der der
 * WorkbenchModel und der WorkbenchView. Ausserdem erzeugt er die
 * Controller-Komponenten von Compiler, Editor, Debugger und Simulation.
 * 
 * Die Werkbank stellt Methoden bereit, ueber die Funktionen von einzelnen
 * Komponenten (Editor, Compiler, ...) aufgerufen werden koennen.
 * 
 * TODO: Classpath einbauen
 * 
 * @author $Author: djasper $
 * @version $Revision: 1.4 $
 */
public class Workbench {
	protected static Workbench workbench;

	/**
	 * Das Model der Werkbank.
	 */
	private WorkbenchModel model;
	/**
	 * Die View der Werkbank.
	 */
	private WorkbenchView view;

	/**
	 * Der Controller des Compilers.
	 */
	private CompilerController compiler;
	/**
	 * Der Controller des Debuggers.
	 */
	private DebuggerController debugger;
	/**
	 * Der Controller des Editors.
	 */
	private EditorController editor;
	/**
	 * Der Controller der Simulation.
	 */
	private SimulationController simulation;

	private Properties settings;

	protected Workbench() {
		settings = new Properties();
		try {
			settings.load(new FileInputStream(Utils.HOME + Utils.FSEP
					+ "settings.properties"));
		} catch (IOException e) {
		}
		model = new WorkbenchModel();

		view = new WorkbenchView(this);

		simulation = new SimulationController(model.getSimulationModel(), this);
		editor = new EditorController(this);
		compiler = new CompilerController(model.getCompilerModel(), this);
		debugger = new DebuggerController(model.getDebuggerModel(), this);

		view.createSimulationFrame();
		view.createEditorFrame();
		view.findMenu("editor", "info").add(
				new JMenuItem(view.showInfoAction));
		model.getSimulationModel().setTerminal(DialogTerminal.getInstance());
		view.setVisible(true);
	}
	public static Workbench getWorkbench() {
		if (workbench == null)
			workbench = new Workbench();
		return workbench;
	}

	/**
	 * Diese Funktion beendet den Hamster-Simulator.
	 */
	public void close(JFrame f) {
		//boolean val = Utils.ask(f, "editor.dialog.quit");
		//if (!val) return;
		if (model.getDebuggerModel().getState() != DebuggerModel.NOT_RUNNING)
			model.getDebuggerModel().stop();
		boolean res = editor.ensureSaved(HamsterFile.getHamsterFile(Utils.HOME));
		if (!res) return;
		try {
			settings.store(new FileOutputStream(Utils.HOME + Utils.FSEP
					+ "settings.properties"), "");
		} catch (IOException e) {
		}
		System.exit(0);
	}

	/**
	 * Diese Funktion hebt eine bestimmte Zeile im Programmtext hervor. Dazu
	 * wird diese im Editor geoeffnet und der Cursor springt zu der
	 * entsprechenden Zeile.
	 * 
	 * @param file
	 *            Die Datei, deren Zeile hervorgehoben werden soll.
	 * @param line
	 *            Die Zeile, die hervorgehoben werden soll.
	 */
	public void markLine(HamsterFile file, int line) {
		editor.markLine(file, line);
	}

	/**
	 * Diese Funktion hebt einen Compiler-Fehler im Programmtext hervor. Dazu
	 * wird die Datei zunaechst im Editor geoeffnet und dann wird an die Stelle
	 * gesprungen, die durch line und column bestimmt ist. Dort wird dann
	 * entsprechend das aktuelle Wort selektiert.
	 * 
	 * @param file
	 *            Die Datei, in der sich der Fehler befindet.
	 * @param line
	 *            Die Zeile, in der sich der Fehler befindet.
	 * @param column
	 *            Die Spalte, in der sich der Fehler befindet.
	 */
	public void markError(HamsterFile file, int line, int column) {
		editor.markError(file, line, column);
	}

	public boolean ensureSaved(HamsterFile file) {
		return editor.ensureSaved(file);
	}
	public boolean ensureCompiled(HamsterFile file) {
		return compiler.ensureCompiled(file);
	}

	/**
	 * Ueber diese Methode benachrichtigt der Editor den Compiler und den
	 * Debugger darueber, dass nun eine andere Datei editiert wird.
	 * 
	 * @param file
	 *            Die nun editierte Datei.
	 */
	public void setActiveFile(HamsterFile file) {
		compiler.setActiveFile(file);
		debugger.setActiveFile(file);
	}

	/**
	 * Diese Methode wird beim Start des Hamster-Simulators aufgerufen.
	 * 
	 * @param args
	 *            Argumente nimmt der Simulator keine entgegen.
	 */
	public static void main(String[] args) {
		Locale.setDefault(Locale.GERMANY);
		// Sicherstellen, dass die noetigen Verzeichnisse existieren, in denen
		// die Hamsterprogramme abgelegt werden.
		Utils.ensureHome();
		Utils.loadProperties(); // dibo

		// Erzeugen der Werkbank.
		getWorkbench();
	}

	/**
	 * Diese Methode liefert den Controller des Compilers.
	 * 
	 * @return Der Controller des Compilers.
	 */
	public CompilerController getComiler() {
		return compiler;
	}

	/**
	 * Diese Methode liefert den Controller des Debuggers.
	 * 
	 * @return Der Controller des Debuggers.
	 */
	public DebuggerController getDebugger() {
		return debugger;
	}

	/**
	 * Diese Methode liefert den Controller des Editors.
	 * 
	 * @return Der Controller des Editors.
	 */
	public EditorController getEditor() {
		return editor;
	}

	/**
	 * Diese Methode liefert den Controller der Simulation.
	 * 
	 * @return Der Controller der Simulation.
	 */
	public SimulationController getSimulation() {
		return simulation;
	}

	/**
	 * Diese Methode liefert das WorkbenchModel.
	 * 
	 * @return Das WorkbenchModel.
	 */
	public WorkbenchModel getModel() {
		return model;
	}

	/**
	 * Diese Methode liefert die WorkbenchView.
	 * 
	 * @return Die WorkbenchView.
	 */
	public WorkbenchView getView() {
		return view;
	}

	public String getProperty(String key, String defaultValue) {
		return settings.getProperty(key, defaultValue);
	}

	public void setProperty(String key, String value) {
		settings.setProperty(key, value);
	}

	public void stop() {
		model.getDebuggerModel().stop();
		DialogTerminal.getInstance().close();
	}
}